!(function ($) {

  var handleBodyRefresh = function (event) {
    if (event.refresh.type !== "after") {
      return;
    }

    if ($(event.refresh.target).closest(".app-body-wrapper").exists()) {
      initIntersectionObserver();
    }
  };

  $(document).ready(function () {
    initIntersectionObserver();
    $(document).on("jalios:refresh", handleBodyRefresh);
  });

  // If we don't have support for intersection observer, load the images immediately
  var initIntersectionObserver = function () {
    lazyLoadImageUrl = $(".explorer-breadcrumb").data(
      "jalios-explorer-lazy-load-image-url"
    );
    isCardDisplayMode =
      $(".explorer-breadcrumb").data("jalios-explorer-display-mode") ===
      "CARDS";
    var $elements = $(".explorer-item:not(.is-lazy-loaded)")
      .not(".explorer-item-category")
      .not(".is-intersection-observed");
    if (!("IntersectionObserver" in window)) {
      // Polyfill
      $elements.each(function (idx, element) {
        loadImage(element);
        $(element)
          .closest(".explorer-item")
          .addClass("is-lazy-loaded");
      });

    } else {
      // It is supported, load the images
      observer = new IntersectionObserver(onIntersection, config);
      $elements.each(function (idx, element) {
        observer.observe(element);
        $(element)
          .closest(".explorer-item")
          .addClass("intersection-observed");
      });
    }
  };

  const config = {
    // If the image gets within 50px in the Y axis, start the download.
    rootMargin: "300px 0px",
    threshold: 0.01
  };

  function onIntersection(elements) {
    // Loop through the entries
    elements.forEach(function (element) {
      // Are we in viewport?
      if (element.intersectionRatio > 0 && !$(element.target).hasClass("is-lazy-loading")) {
        // Stop watching and load the image
        observer.unobserve(element.target);

        loadImage(element.target, function () {
          $(element.target)
            .closest(".explorer-item")
            .addClass("is-lazy-loaded")
            .removeClass("is-lazy-loading")
        });
      }
    });
  }

  var loadImage = function (element, callback) {

    $.ajax({
      url: lazyLoadImageUrl,
      data: {
        cardData: jQuery(element)
          .closest(".explorer-item")
          .data("jalios-data-id"),
        loadThumb: true
      },
      type: "post"
    }).done(function (html) {
      if (isCardDisplayMode) {
        let $card = jQuery(element).closest(".card-publication");
        $card.find(".card-placeholder").after(html);
        setTimeout(function () {
          $card.find(".loaded-thumb").addClass("loaded-thumb-show");
        }, 1);
        $card.find(".card-placeholder:not(.loaded-thumb)").remove();
        var imgUrl = $card.find(".card-img IMG").prop("src");
        if ("objectFit" in document.documentElement.style === false) {
          if (imgUrl && !$card.hasClass("object-fit-compat")) {
            $card.find(".card-img IMG")
              .css("backgroundImage", "url(" + imgUrl + ")")
              .addClass("object-fit-compat")
              .attr("alt", "")
              .attr("src", "");
          }
        }

      } else {
        let $card = jQuery(element).closest(".explorer-table-row");
        $card.find(".list-image-placeholder").after(html);
        setTimeout(function () {
          $card.find(".loaded-thumb").addClass("loaded-thumb-show");
        }, 1);
        $card.find(".list-image-placeholder:not(.loaded-thumb)").remove();
      }
      if (callback) {
        callback();
      }
    });
  };

})(window.jQuery);
