!(function ($) {
  // TODO
  // * Remove section if empty after removeFunctions have been called

  // Keys
  var IS_CTRL_KEY_PRESSED = false;
  const CTRL_KEY = 17;
  var IS_SHIFT_KEY_PRESSED = false;
  const SHIFT_KEY = 16;
  const A_KEY = 65;
  const ESCAPE_KEY = 27;
  const A_MIN_KEY = 97;

  // Urls
  const ACTION_URL = 'plugins/ExplorerPlugin/jsp/action/handleExplorerAction.jsp';
  const CTXT_MENU_URL = 'plugins/ExplorerPlugin/jsp/app/contextmenu/doExplorerContextMenu.jsp ';
  const CATEGORY_NODE_CHILDRENS_URL = 'plugins/ExplorerPlugin/jsp/app/doExplorerAppSidebar_categoryNodeChildrens.jsp';
  const EDIT_SIDEBAR_URL = 'plugins/ExplorerPlugin/jsp/app/editSidebar/doExplorerEditSidebarInner.jsp';

  // Elements
  var $allContextMenus = $('.explorer-context-menu');
  var $explorerItemData;
  var $draggedItem;
  var $lastActiveItem;

  // Objects
  var EXPLORER;
  var EXPLORER_PREVIEW_SIDEBAR;
  var EXPLORER_EDIT_SIDEBAR;

  // references
  var sidebarPublicationId;
  var EXPLORER_CTX_MENU_ITEM_ID;
  const initialPageTitle = document.title;

  var lazyLoadImageUrl = '';
  var isCardDisplayMode;
  // Other
  var ajaxChangeColorTimeout;

  // Global variable for sidebar tab when selecting item since we cannot retrieve options in sidebar opening event
  let sidebarTab = "";

  // ------------------------------------------
  //  Objects
  // ------------------------------------------

  const initExplorer = function () {
    EXPLORER = new Explorer($('.app-explorer'));
    if (EXPLORER.debugMode) {
      consoleDebug('Debug mode enabled');
    }
    initDisplayedPublication();
  };

  const initExplorerFilterEvents = function () {
    $(document).on(
      'input change propertychange datechange',
      '.js-explorer-category-chooser .jalios-category',
      function (event) {
        //EXPLORER.selectItem(null, false, false, $(event.currentTarget).val());
        var itemId = $(event.currentTarget).val();

        var openedCategories = ExplorerUtils.getOpenedSidebarCategories();
        openedCategories.push(itemId);

        const currentExplorer = $.plugin.ExplorerApp.getExplorer(event.currentTarget);

        let params = {
          explorerCurrentCategory: currentExplorer.data.currentCategoryId,
          portletExplorer: currentExplorer.data.portletExplorerId,
          sidebarOpenedCategories: openedCategories
        }

        $('.app-sidebar-categorytree').refresh({
          nohistory: true,
          params: params,
          callback: function () {
            currentExplorer.selectItem($('.sidebar-categorytree-item [data-jalios-data-id=' + itemId + ']'), false, false);
            currentExplorer.focusBreadcrumb();
          }
        });
      }
    );

    $(document).on(
      'input change propertychange datechange jalios.date.change',
      '[data-jalios-explorer-filter]',
      function (event) {
        event.stopImmediatePropagation();
        event.stopPropagation();
        ExplorerUtils.toggleExplorerLoading();

        var formData = ExplorerUtils.getExplorerFormData();
        formData.append('isFiltering', true);
        var object = ExplorerUtils.getFormDataAsJsonObject(formData);
        const currentExplorer = $.plugin.ExplorerApp.getExplorer(event.currentTarget);
        let $refreshedElement =
          ExplorerUtils.getDisplayMode() === 'LIST' ? $('.explorer-results') : $('.explorer-section-publications');

        $refreshedElement.refresh({
          nohistory: true,
          isform: true,
          callback: function () {
            ExplorerUtils.hideExplorerLoading();
            let isFilteredTypeDocument = $('.js-explorer-is-filtered-type-document').val() === 'true';
            if (isFilteredTypeDocument) {
              $('.kind-filter SELECT').prop('disabled', false);
            } else {
              $('.kind-filter SELECT').prop('disabled', true);
            }
            currentExplorer.focusBreadcrumb();

          },
          params: object
        });
      }
    );
  };

  const initExplorerKeyboardEvents = function () {
    $(document).on('keydown', function (event) {
      if (event.keyCode === ESCAPE_KEY) {
        EXPLORER_PREVIEW_SIDEBAR.close({});
        return;
      }

      // Do not perform select all if a modal is opened
      let $excludedComponents = $('#jalios-modal.in, .sidebar-component.is-open');
      if ($excludedComponents.exists()) {
        return;
      }

      // Check an input is not focused
      if ($(document.activeElement).is(':input')) {
        return;
      }

      if (event.ctrlKey && (event.keyCode === A_KEY || event.keyCode === A_MIN_KEY)) {
        event.preventDefault();
        // Select all
        EXPLORER.selectAll();
      }

    });
  };

   const isFocusedClick = function(event){
    if(event.type === 'click'){
      return true;
    }
    else if(event.type === 'keydown'){
      let code = event.charCode || event.keyCode;
      if((code === 32)|| (code === 13)){
        return true;
      }
    }
    else{
      return false;
    }
  }

  const initExplorerClickEvents = function () {
    // Handle conflict when there's multiple selected items (active) and a single item menu has been opened.
    $(document).on('show.bs.dropdown', function(event) {
      if ($(event.target).is(".explorer-actions-dropdown") && $('.explorer-item.active').length > 1 && !$(event.target).find(".multiple-items").exists()) {
        updateActionsMenu();
      }
    })

    $(document).on('click', '.breadcrumb-item', function (event) {
      event.preventDefault();

      if ($(event.currentTarget).hasClass('is-search')) {
        return;
      }

      EXPLORER.selectItem($(event.currentTarget), event.ctrlKey, event.shiftKey);

      //ExplorerUtils.toggleExplorerLoading();
      // $(".app-body").refresh({
      //   params: {
      //     portletExplorer: EXPLORER.data.portletExplorerId,
      //     explorerCurrentCategory: $(event.currentTarget).data(
      //       "jalios-explorer-target-category"
      //     )
      //   },
      //   callback: function() {
      //     ExplorerUtils.hideExplorerLoading();
      //   }
      // });
    });

    $(document).on('click keydown', '.js-toggle-sidebar-category-search', function (event) {
      if (!isFocusedClick(event)) {
        return;
      }
      $('.category-search-field').toggleClass('hide');
      
      let expanded = JSON.parse($(event.currentTarget).attr("aria-expanded"));
      $(event.currentTarget).attr("aria-expanded", !expanded);

      $.jalios.ui.Widget.smartFocus($('.category-search-field .form-control-autocomplete'));
    });

    $(document).on('click', function (event) {
      if (!$(event.target).closest('#contextMenu').exists()) {
        $allContextMenus.css('display', 'none');
      }
    });

/*    $(document).on('click', '.app-explorer', function (event) {
      // Remove active class TODO : add method for this
      // $(".explorer-item").removeClass("active");
      // $(".checkbox-check").prop("checked", false);
      ExplorerUtils.removeCategoryItemsFromSelection();
      ExplorerUtils.removePublicationItemsFromSelection();

      // Reset global actions menu
      updateActionsMenu();
    });*/

    // Disable browser navigation when clicking on a publication card (Open is performed in sidebar)
    $(document).on('click', '.explorer-item.card-publication', function (event) {
      event.preventDefault();
      event.stopPropagation();
    });

    $(document).on('click keydown', '.js-activate-card, .checkbox-wrapper', function (event) {
      if (!isFocusedClick(event)) {
        return;
      }
      event.stopImmediatePropagation();
      event.stopPropagation();
      event.preventDefault();
      
      const currentExplorer = $.plugin.ExplorerApp.getExplorer(event.currentTarget);
      
      let $item = $(event.currentTarget).closest('.explorer-item');

      if ($item.hasClass('active')) {
        $item.find('.js-activate-card').attr('data-original-title', $item.find(".js-activate-card").data("jalios-select-title"));
        $item.find('.checkbox-check').prop('checked', false);
        $item.removeClass('active');
        $item.attr('aria-selected', false);
        updateActionsMenu();
        currentExplorer.updateSelectedElementsNavbarItem();
      } else {
        $item.find('.js-activate-card').attr('data-original-title', $item.find(".js-activate-card").data("jalios-unselect-title"));
        $(event.currentTarget).find('.checkbox-check').prop('checked', true);
        ExplorerUtils.addItemToSelection(
          $(event.currentTarget).closest('.explorer-item'),
          true,
          event.ctrlKey,
          event.shiftKey
        );
      }
    });

    $(document).on('change', '.js-activate-card INPUT:checkbox', function (event) {
      event.stopImmediatePropagation();
      event.stopPropagation();
      ExplorerUtils.addItemToSelection(
        $(event.currentTarget).closest('.explorer-item'),
        true,
        event.ctrlKey,
        event.shiftKey
      );
    });

    // ------------------------------------------
    //  Contextual menus (Right click)
    // ------------------------------------------

    // Close all menus when a click is performed inside one of these menus
    $allContextMenus.on('click', 'a', function () {
      $allContextMenus.hide();
    });

    // Right click on explorer items
    $(document).on(
      'contextmenu',
      '.explorer-row-item, .explorer-card-item, .sidebar-categorytree-item, .breadcrumb-item',
      function (e) {
        if ($(e.currentTarget).hasClass('breadcrumb-item') && $(e.currentTarget).data('jalios-data-id') === '') {
          return;
        }
        var $card = $(e.currentTarget);
        updateCardContextMenu($card, e);
        return false;
      }
    );

    // Disable right click default browser behavior
    $('.explorer-item').mousedown(function (event) {
      event.preventDefault();
    });

    $(document).on('mouseup', '.explorer-table-body [data-jalios-explorer-action=selectItem]', function (event) {
      switch (event.which) {
        case 2:
          let url = $(event.currentTarget).data('jalios-data-url');
          if (url) {
            var win = window.open(url, '_blank');
            if (win) {
              break;
            }
            return true;
          }
      }
    });

    $(document).on('click keydown', '[data-jalios-explorer-action]', function (event) {
      if (!isFocusedClick(event)) {
        return;
      }
      let $item = $(event.currentTarget);
      // Prevent click in table :
      if ($(event.target).hasClass('is-action')) {
        $(event.target).find('A')[0].click();
        return;
      }

      if ($(event.target).closest('.is-action').exists()) {
        $(event.target).closest('.is-action')[0].click();
        return;
      }

      event.stopPropagation();
      event.preventDefault();

      const currentExplorer = $.plugin.ExplorerApp.getExplorer(event.currentTarget);

      var action = $(event.currentTarget).data('jalios-explorer-action');
      let dataId = $(event.currentTarget).data('jalios-data-id');
      switch (action) {
        case 'unselectAllItems': 
          currentExplorer.removeAllSelectedItems();
          updateActionsMenu();
          currentExplorer.updateSelectedElementsNavbarItem();
          break;
        case 'openActionsMenu': 
          //currentExplorer.removeAllSelectedItems();
          $.fn.dropdown.clearMenus();
          updateActionsMenu($item.closest(".explorer-item"));
          //currentExplorer.updateSelectedElementsNavbarItem();
          updateCardContextMenu($item.closest(".explorer-item"), event, true);
          break;        
        case 'toggleDisplayMode': 
          consoleDebug('Action [toggleDisplayMode]');
          EXPLORER.toggleDisplayMode();
          break;
        case 'deleteMultiple':
          consoleDebug('Action [delete] on ' + EXPLORER_CTX_MENU_ITEM_ID);
          EXPLORER.deleteItems();
          break;
        case 'delete':
          consoleDebug('Action [delete] on ' + EXPLORER_CTX_MENU_ITEM_ID);
          EXPLORER.deleteItem(EXPLORER_CTX_MENU_ITEM_ID, $item.data('jalios-data-name'));
          break;
        case 'selectAll':
          EXPLORER.selectAll();
          break;
        case 'changeCategoryColor':
          const $colorPicker = $(".explorerChangeColorPicker");
          $colorPicker.spectrum("toggle");
          $colorPicker.data("colorDataId", dataId);
          
          const $spectrumContainer = $(".sp-container");
          
          let $currentItem = $('.explorer-item[data-jalios-data-id=' + dataId + ']');
          if ($currentItem.is('.explorer-table-row')) {
            $spectrumContainer.appendTo($currentItem.find('.item-title'));
          } else {
            $spectrumContainer.appendTo($currentItem);
          }
          $spectrumContainer.one('click', function (event) {
            event.stopPropagation();
            event.stopImmediatePropagation();
          });
          
          $colorPicker.one('change.spectrum', function(e, tinycolor) { 
            const dataId = $colorPicker.data("colorDataId");
            $('.sidebar-categorytree-item[data-jalios-data-id=' + dataId + '] .jcmsplugin-explorer-folder').css("color", tinycolor.currentVal);
            $('.explorer-item[data-jalios-data-id=' + dataId + '] .item-title .jcmsplugin-explorer-folder').css("color", tinycolor.currentVal);
            consoleDebug('Action [CHANGE COLOR] on ' + tinycolor.currentVal);
            if (ajaxChangeColorTimeout) {
              clearTimeout(ajaxChangeColorTimeout);
            }
            ajaxChangeColorTimeout = setTimeout(function () {
              $.ajax({
                url: ACTION_URL,
                explorer: currentExplorer,
                data: {
                  targetCategory: dataId,
                  color: tinycolor.currentVal,
                  opChangeCategoryColor: true
                },
                type: 'post'
              }).done(function (data) {
                toastr.clear();
                ExplorerUtils.displayMessage(data.message, data.actionStatus === 'ERROR');
                ajaxChangeColorTimeout = false;
              });
            }, 500);
          });
          break;
        case 'unlockPublication':
        case 'lockPublication':
          var isLock = $(event.currentTarget).data('jalios-explorer-action') === 'lockPublication';
          var ajaxData = isLock ? { opLock: true, items: dataId } : { opUnlock: true, items: dataId };
          $.ajax({
            url: ACTION_URL,
            explorer: currentExplorer,
            data: ajaxData,
            type: 'post'
          })
            .done(function (data) {
              if (isLock) {
                $('[data-jalios-data-id=' + dataId + '] .is-strong-locked').removeClass('hide');
              } else {
                $('[data-jalios-data-id=' + dataId + '] .is-strong-locked').addClass('hide');
              }
              ExplorerUtils.displayMessage(data.message, data.actionStatus === 'ERROR');
            })
            .fail(function () {
              //TODO handle fail (Server error)
            });
          break;
        case 'removeCategory':
          consoleDebug('Action [removeCategory] on ' + EXPLORER_CTX_MENU_ITEM_ID);
          EXPLORER.removeItemFromCategory(EXPLORER_CTX_MENU_ITEM_ID);
          break;
        case 'addCategoryToFilters':
          consoleDebug('Action [addCategoryToFilters] on ' + EXPLORER_CTX_MENU_ITEM_ID);

          if ($('.category-filter-item INPUT[value=' + dataId + ']').exists()) {
            ExplorerUtils.displayMessage(explorerAppI18Nprops.alreadyPinnedWarning);
            return;
          }

          var $clonedItem = $('.category-filters .category-filter-item-sample').clone();
          $clonedItem.find(' > INPUT:first').val(dataId);
          $clonedItem.find(' > INPUT:first').removeAttr('disabled');
          var categoryName = $(event.currentTarget).data('jalios-category-name')
            ? $(event.currentTarget).data('jalios-category-name')
            : $('.explorer-item[data-jalios-data-id=' + dataId + '] .js-title').text();
          var textNode = document.createTextNode(categoryName);
          $clonedItem.prepend(textNode);
          $clonedItem.removeClass('category-filter-item-sample').removeClass('hide');
          $clonedItem.appendTo($('.category-filters'));

          var formData = ExplorerUtils.getExplorerFormData();
          formData.append('isFiltering', true);
          var object = ExplorerUtils.getFormDataAsJsonObject(formData);

          $('.explorer-section-publications').refresh({
            nohistory: true,
            isform: true,
            callback: function () {
              ExplorerUtils.hideExplorerLoading();
            },
            params: object
          });

          break;
        case 'removeCategoryFromFilters':
          consoleDebug('Action [removeCategoryFromFilters]');
          $item.closest('.category-filter-item').remove();

          var formData = ExplorerUtils.getExplorerFormData();
          formData.append('isFiltering', true);
          var object = ExplorerUtils.getFormDataAsJsonObject(formData);

          $('.explorer-section-publications').refresh({
            isform: true,
            nohistory: true,
            callback: function () {
              ExplorerUtils.hideExplorerLoading();
            },
            params: object
          });

          break;
        case 'editData':
          consoleDebug('Action [editData] on ' + EXPLORER_CTX_MENU_ITEM_ID);
          EXPLORER_EDIT_SIDEBAR.open({});
          break;
        case 'renameData':
          consoleDebug('Action [renameData] on ' + EXPLORER_CTX_MENU_ITEM_ID);
          $.jalios.ui.Modal.prompt(
            $(event.currentTarget).data('jalios-text'),
            function (newName) {
              if (newName === false) {
                return;
              }

              $.ajax({
                url: ACTION_URL,
                explorer: currentExplorer,
                data: {
                  items: dataId,
                  opRenameData: true,
                  dataName: newName
                },
                type: 'post'
              })
                .done(function (data) {
                  // Change name of sidebar Items & card
                  $('[data-jalios-data-id=' + dataId + ']').each(function(idx, element) {
                    if ($(element).closest(".sidebar-categorytree-item").exists()) {
                      let $img = $('> .sidebar-categorytree-item-link .js-title .jalios-icon', element);
                      $('> .sidebar-categorytree-item-link .js-title', element).text(' ' + newName);
                      $('> .sidebar-categorytree-item-link .js-title', element).prepend($img);
                    } else {
                      let $img = $('[data-jalios-data-id=' + dataId + '] .js-title .jalios-icon');
                      $('.js-title', element).text(' ' + newName);
                      $('.js-title', element).prepend($img);
                    }
                  });

                  ExplorerUtils.displayMessage(data.message, data.actionStatus === 'ERROR');
                })
                .fail(function () {
                  //TODO handle fail (Server error)
                });
            },
            $.jalios.Http.encode($(event.currentTarget).data('jalios-current-name'))
          );
          break;
        case 'openPublicationPreviewOnRightsTab':
          sidebarTab = "rights";
          EXPLORER.selectItem($item, event.ctrlKey, event.shiftKey, dataId);
          break;
        case 'selectItem':
          var itemId = $item.data('jalios-data-id');
          consoleDebugGroup('Action [selectItem] on ' + itemId, $item);
          EXPLORER_PREVIEW_SIDEBAR.close({});
          
          if ($(event.target).closest(".btn-check-publication-rights").exists()) {
            sidebarTab = "rights";
          } else {
            sidebarTab = "";
          }
          
          EXPLORER.selectItem($item, event.ctrlKey, event.shiftKey);
          break;
        case 'spellSuggestion':
          let $searchInput = EXPLORER.$element.find('.explorer-search-input');
          $searchInput.val($item.text());
          $searchInput.refresh();
          break;
        case 'sort':
          let $header = $item.closest('.explorer-table-header');
          let $sortSelect = EXPLORER.$element.find('select[name=explorerSort]');
          $sortSelect.val($item.data('jalios-explorer-sort'));
          let isActive = $header.hasClass('active');
          EXPLORER.$element.find('.explorer-table-header').removeClass('active');
          $header.removeClass('active');

          let $sortDescendingState = EXPLORER.$element.find('input[name=explorerSortDescending]');
          let isDescending = JSON.parse(EXPLORER.$element.find('input[name=explorerSortDescending]:checked').val());

          if (isActive) {
            $header.addClass('active');
            $header.toggleClass('is-descending');
            if (isDescending) {
              EXPLORER.$element.find('.explorer-sort-descending-enabled').prop('checked', false);
              EXPLORER.$element.find('.explorer-sort-descending-disabled').prop('checked', true);
            } else {
              EXPLORER.$element.find('.explorer-sort-descending-enabled').prop('checked', true);
              EXPLORER.$element.find('.explorer-sort-descending-disabled').prop('checked', false);
            }
          } else {
            $header.addClass('active');
            EXPLORER.$element.find('.explorer-sort-descending-enabled').prop('checked', true);
            EXPLORER.$element.find('.explorer-sort-descending-disabled').prop('checked', false);
          }

          $sortSelect.change();
          break;
        case 'toggleSidebarCategory':
          var $categoryTreeItem = $item.closest('.sidebar-categorytree-item');
          consoleDebugGroup('Action [toggleSidebarCategory] ' + itemId, $categoryTreeItem);

          $categoryTreeItem.toggleClass('is-opened');
          if ($categoryTreeItem.attr('aria-expanded') === 'true') {
            $categoryTreeItem.attr('aria-expanded', false);
          } else {
            $categoryTreeItem.attr('aria-expanded', true);
          }
          $categoryTreeItem.find('> A .sidebar-categorytree-item-opener .opener-icon').toggleClass('hide');

          var $subMenu = $categoryTreeItem.find('> .app-sidebar-menu');
          // if menu is already opened, do not perform a new ajax query to display current node childrens
          if (!$subMenu.exists()) {
            var categoryItemId = $categoryTreeItem.data('jalios-data-id');
            $.ajax({
              url: CATEGORY_NODE_CHILDRENS_URL,
              explorer: currentExplorer,
              data: {
                openedCategoryId: categoryItemId,
                explorerCurrentCategory: EXPLORER.data.currentCategoryId,
                portletExplorer: EXPLORER.data.portletExplorerId
              },
              type: 'post'
            }).done(function (html) {
              $(html).appendTo($categoryTreeItem);
              initDroppable('.sidebar-categorytree-item:not(.is-root-access) > A');
              initDraggable('.sidebar-categorytree-item.is-usable');
              $.plugin.ExplorerApp.triggerEvent('openedCategory');
            });
          }

          break;
        default:
          // Trigger an event
          consoleDebug('Custom action [' + action + '], event is triggered');
          $.plugin.ExplorerApp.triggerEvent('action', {
            $item: $item,
            dataId: $(event.currentTarget).data('jalios-data-id'),
            currentCategory: EXPLORER.data.currentCategoryId,
            portletExplorer: EXPLORER.data.portletExplorerId
          });
      }
    });
  };

  /**
   * The explorer component
   * @constructor
   * @typedef {Object} Explorer
   * @param {HTMLElement} element - the element containing the explorer
   */
  const Explorer = function (element) {
    this.$element = $(element);
    this.$element.Explorer = this;
    this.data = {};
    this.debugMode = false;
    this.init();
  };

  Explorer.prototype.updateData = function () {
    this.data.currentCategoryId = $('.js-explorer-current-category').attr('value');
  };

  Explorer.prototype.init = function () {
    this.isDragAndDropAvailable = $(this.$element).data('jalios-explorer-dnd-available') === true;
    this.$element.data('explorer', this);
    this.data.portletExplorerId = $(this.$element).data('jalios-explorer-portlet-id');

    consoleDebug('Initializing Explorer object on portlet ' + this.data.portletExplorerId);

    this.updateData();
    initExplorerClickEvents();
    initExplorerFilterEvents();
    initExplorerKeyboardEvents();
    initDraggableAndDroppable(this);
    initSidebar();
    initEventHandling(this);
    $('.sidebar-component').removeClass('hide');

    // Hide popover content when a modal is opened
    $(document).on("jalios:refresh", function(event) {
      if (event.refresh.type !== "after") {
        return;
      }
      
      if ($(event.refresh.target).exists()) {
        if ($(event.refresh.target).hasClass("modal")) {
          setTimeout(function(){
            $(".popover").removeClass("in");
          }, 500);
        }
        if ($(event.refresh.options.brokerTrigger).hasClass("see-more-btn")) {
          // Update total row count on table view
          const currentExplorer = $.plugin.ExplorerApp.getExplorer($(event.refresh.options.brokerTrigger));
          if ($(".explorer-table-body.explorer-section-publications", currentExplorer.$element).exists()) {
            const rowsCount = $(".explorer-table-body.explorer-section-publications .explorer-table-row:last").attr("aria-rowindex")
            $(".explorer-table", currentExplorer.$element).attr("aria-rowcount", rowsCount);
          }
        }
      } 
    });
    
    $(".sidebar-component").removeClass("hide");

    // Init active category in sidebar
    if (!this.isUnclassifiedFilterEnabled()) {
      $('.sidebar-categorytree-item[data-jalios-data-id=' + this.data.currentCategoryId + ']').addClass('active').find("> A").attr("aria-current", "true");
    }

    // Hide dropdowns when any dropdown opens
    $(document).on('show.bs.dropdown', function () {
      $allContextMenus.css('display', 'none');
    });

    $(document).on('scroll', function () {
      // When scrolling, remove all context $allContextMenus
      $allContextMenus.hide();
      // Also remove dropdown in explorer navbar
      $('.dropdown.open .dropdown-toggle')
        .filter(function (index, element) {
          return !$(element).is('#explorer-actions-menu') && !$(element).is('#explorer-add-menu');
        })
        .dropdown('toggle');
    });
    this.debugMode = $('.app-body').data('jalios-explorer-debug-mode-enabled');
  };

  Explorer.prototype.initAfterAjax = function () {
    this.$element = $('.app-explorer');
    this.$element.data('explorer', this);
    initDraggableAndDroppable(this);
    //updateCardContextMenu();
    this.updateData();
  };

  /**
   * Selects all publication displayed
   * @name Explorer#selectAll
   * @function
   */
  Explorer.prototype.selectAll = function () {
    ExplorerUtils.removeCategoryItemsFromSelection();
    let $items = $('.explorer-item.explorer-item-data');

    if ($items.length === 0) {
      ExplorerUtils.displayMessage(explorerAppI18Nprops.noPublicationToSelect);
      return;
    }
    $items.each(function (index, element) {
      let $element = $(element);

      // Toggle select/unselect label
      if (!$element.hasClass('active')) {
        $element.find('.js-activate-card').attr('data-original-title', $element.find(".js-activate-card").data("data-jalios-unselect-title"));
      }

      $element.addClass('active');
      $element.find('.checkbox-check').prop('checked', true);
    });
    updateActionsMenu();
    this.updateSelectedElementsNavbarItem();
    ExplorerUtils.displayMessage($.jalios.I18N.glp('jcmsplugin.explorer.app.info.select-all', $items.length));
  };

  /**
 * Ajax-refresh on the full list of categories in sidebar
 * @name Explorer#refreshFullCategoryListInSidebar
 * @function
 */
  Explorer.prototype.refreshFullCategoryListInSidebar = function () {
    let $sidebarTree = $(this.$element).find(".app-sidebar-categorytree");

    if (!$sidebarTree.exists()) {
      return;
    }

    $sidebarTree.removeData('ajax-refresh:job')
    $sidebarTree.refresh({
      params: {
        portletExplorer: EXPLORER.data.portletExplorerId
      },
      nohistory: true,
      noscroll: true,
      callback: function () {
        $('.sidebar-categorytree-item').removeClass("active");
        $('.sidebar-categorytree-item[data-jalios-data-id=' + EXPLORER.data.currentCategoryId + ']').addClass('active').find("> A").attr("aria-current", "true");
      }
    });
  }

  Explorer.prototype.isListView = function() {
    return this.$element.find(".explorer-table-wrapper").exists();
  }


  Explorer.prototype.updateItemNumbers = function() {
    if (!this.isListView()) {
      return;
    }
    
    let $currentExplorer = this.$element;
    
    let $items = $currentExplorer.find(".explorer-table-body .explorer-table-row");
    let totalItems = $items.length;
    
    // Update total row count on table
    $currentExplorer.find(".explorer-table").attr("aria-rowcount", totalItems);
    
    let itemIndex = 1;
    
    // Update index column and aria
    $items.each(function(idx, element) {
      $(element).find(".explorer-table-col-_cnt_").text(itemIndex);
      $(element).attr("aria-rowindex", itemIndex);
      itemIndex = itemIndex +1;
    });
  }
  
  /**
   * Ajax-refresh on the category list
   * @name Explorer#refreshCategoryList
   * @function
   */
  Explorer.prototype.refreshCategoryList = function (categoryCreatedIds) {
    $(this.$element)
      .find('.explorer-section-categories')
      .refresh({
        nohistory: true,
        params: {
          explorerCurrentCategory: EXPLORER.data.currentCategoryId,
          portletExplorer: EXPLORER.data.portletExplorerId
        },
        callback: function (event) {
          // Handle table refresh
          var categoryCount = $('.explorer-table .explorer-item-category').length;
          if (categoryCount > 0) {
            $('.explorer-table').removeClass('hide');
          } else {
            $('.explorer-table').addClass('hide');
          }
          // Highlight created categories
          categoryCreatedIds.forEach(function (categoryCreatedId) {
            $('.explorer-item[data-jalios-data-id=' + categoryCreatedId + ']').addClass('has-been-added');
          });

          // Scroll to first created category
          $([document.documentElement, document.body]).animate(
            {
              scrollTop:
                $('.explorer-item[data-jalios-data-id=' + categoryCreatedIds[0] + ']').offset().top -
                $('.topbar').height() -
                20
            },
            500
          );
        }
      });
  };

  /**
   * Returns true if current portlet is displayed as collaborative space application
   * @name Explorer#isCollaborativeSpaceApplicationDisplay
   * @function
   */
  Explorer.prototype.isCollaborativeSpaceApplicationDisplay = function() {
    return this.$element.find("input[name=collaborativeSpaceApplicationDisplay]").val();
  }
  
    /**
   * Returns true if current portlet is displayed as collaborative space application
   * @name Explorer#isCollaborativeSpaceApplicationDisplay
   * @function
   */
  Explorer.prototype.isSelectionEnabled = function() {
    return this.$element.find(".explorer-result-wrapper.is-selection-enabled").exists();
  }

  /**
   * Displays no result element if no element is found
   * @name Explorer#refreshNoResultDisplay
   * @function
   */
  Explorer.prototype.refreshNoResultDisplay = function () {
    if (ExplorerUtils.getDisplayMode() === 'CARDS') {
      if ($('.explorer-results .card').size() === 0) {
        $('.app-explorer-no-results').removeClass('hide');
      } else {
        $('.app-explorer-no-results').addClass('hide');
      }
    } else {
      if ($('.explorer-results .explorer-table-row').size() <= 1) {
        $('.app-explorer-no-results').removeClass('hide');
        $('.explorer-table').addClass('hide');
      } else {
        $('.app-explorer-no-results').addClass('hide');
        $('.explorer-table').removeClass('hide');
      }
    }

    // Clone dropdown
  };

  Explorer.prototype.updateSelectedElementsNavbarItem = function () {
    const $activeItems = $('.explorer-item.active');
    const activeItemIds = ExplorerUtils.getItemIds($activeItems);
    
    if (activeItemIds.size() > 0) {
      this.$element.find(".explorer-result-wrapper").addClass("is-selection-enabled");
    } else {
      this.$element.find(".explorer-result-wrapper").removeClass("is-selection-enabled");
    }
    if (activeItemIds.size() > 1) {
      this.$element.find(".navbar-selected-elements .number").text(activeItemIds.size());
      this.$element.find(".navbar-selected-elements").removeClass("hide");
    } else {
      this.$element.find(".navbar-selected-elements").addClass("hide");
    }
  }

  /**
   * PerformDelete on item (Publication or Category)
   * @name Explorer#deleteItem
   * @param {String} itemId
   * @function
   */
  Explorer.prototype.deleteItem = function (itemId, itemName) {
    const currentExplorer = this;
    let deleteMessage = $.jalios.I18N.glp('jcmsplugin.explorer.app.confirm.delete', itemName);
    $.jalios.ui.Modal.confirm(deleteMessage, function () {
      $.ajax({
        url: ACTION_URL,
        explorer: currentExplorer,
        data: {
          items: itemId,
          opDelete: true
        },
        type: 'post'
      })
        .done(function (data) {
          if (data.actionStatus === 'OK') {
            //TODO update category tree
            var $deletedElement = $('.explorer-item[data-jalios-data-id=' + itemId + ']');
            EXPLORER.removeItemByElement($deletedElement);

            //TODO : improve use a unique method to remove an item by its id (Tree / Categories in body, and handling of last category item)
            if (ExplorerUtils.getDisplayMode() === 'CARDS') {
              if ($('.explorer-section-categories .card').length === 0) {
                $('.explorer-section-categories').html('');
              }
            } else if (ExplorerUtils.getDisplayMode() === 'LIST') {
            }

            $('.sidebar-categorytree-item[data-jalios-data-id=' + itemId + ']').remove();
            //TODO handle last publication removed

            ExplorerUtils.displayMessage(data.message, data.actionStatus === 'ERROR');
            EXPLORER.refreshNoResultDisplay();
            //ExplorerUtils.handleTableDisplay();
          }
        })
        .fail(function () {
          //TODO handle fail (Server error)
        });
    });
  };

  /**
   * PerformDelete on multiple items (Publications)
   * @name Explorer#deleteItems
   * @function
   */
  Explorer.prototype.deleteItems = function (itemId) {
    var $activeItems = $('.explorer-item.active');
    const currentExplorer = this;
    $.ajax({
      url: ACTION_URL,
      explorer: currentExplorer,
      data: {
        items: ExplorerUtils.getItemIds($activeItems),
        opDelete: true
      },
      traditional: true,
      type: 'post'
    })
      .done(function (data) {
        if (data.actionStatus === 'OK') {
          Object.values(data.data.removedItemIds).forEach(function (removedItemId) {
            // TODO : remove this fugly fix when prototype will be removed for god sake
            if (!(typeof removedItemId === "function")) {
              var $deletedElement = $('.explorer-item[data-jalios-data-id=' + removedItemId + ']');
              EXPLORER.removeItemByElement($deletedElement);
            }
          });
          ExplorerUtils.displayMessage(data.message, data.actionStatus === 'ERROR');
          EXPLORER.refreshNoResultDisplay();
          //ExplorerUtils.handleTableDisplay();
        } else if (data.actionStatus === 'FORBIDDEN') {
          ExplorerUtils.displayMessage(data.message, data.actionStatus === 'FORBIDDEN');
        }
      })
      .fail(function () {
        //TODO handle fail (Server error)
      });
  };

/**
   * Switch between cards / list display
   * @name Explorer#toggleDisplayMode
   * @function
   */
  Explorer.prototype.toggleDisplayMode = function () {
    const explorer = this;
    
    const $displayModeButton = $(this.$element).find(".display-mode-chooser");
    const previousVal = $displayModeButton.find("input[name=displayMode]").val();
    const newVal = previousVal === "CARDS" ? "LIST" : "CARDS";
    const previousTitle = $displayModeButton.attr("title");
    
    $displayModeButton.find("input[name=displayMode]").val(newVal);
    $displayModeButton.find(".jalios-icon").addClass("hide")
    $displayModeButton.find(".label-wrapper > SPAN").addClass("hide");
    $displayModeButton.find(".display-mode-" + previousVal).removeClass("hide")
    $displayModeButton.attr("title", $displayModeButton.data("jalios-alt-title"));
    $displayModeButton.data("jalios-alt-title", previousTitle);

    ExplorerUtils.toggleExplorerLoading();

    $('.explorer-results').refresh({
      nohistory: true,
      params: ExplorerUtils.getExplorerFormDataAsJsonObject(true),
      callback: function () {
        ExplorerUtils.hideExplorerLoading();
        saveDisplayModePreference(explorer);
      }
    });
  };
  
  const saveDisplayModePreference = function(explorer) {
    $.ajax({
      url: ACTION_URL,
      explorer: explorer,
      data: {
        displayMode: explorer.$element.find(".display-mode-chooser input[name=displayMode]").val(),
        portletExplorer: explorer.data.portletExplorerId,
        opSavePreferences: true
      },
      callback: function() {
      },
      type: 'post'
    }).done(function (data) { });
  }

  /**
   * Removes element (Publication or Category) from the DOM
   * @name Explorer#removeItemByElement
   * @param {jQuery} $element
   * @function
   */
  Explorer.prototype.removeItemByElement = function ($element) {
    if ($element.closest('.card-link').exists()) {
      $element.closest('.card-link').remove();
    } else {
      $element.remove();
    }
    
    this.updateItemNumbers();
  };

  /**
   * Removes item (Publication or Category) from the DOM by its Id
   * @name Explorer#removeItemById
   * @param {String} elementId
   * @function
   */
  Explorer.prototype.removeItemById = function (elementId) {
    var $removedElement = $('[data-jalios-data-id=' + elementId + ']');
    if ($removedElement.closest('.card-link').exists()) {
      $removedElement.closest('.card-link').remove();
    } else {
      $removedElement.remove();
    }
    this.updateItemNumbers();
  };

  /**
   * Removes item (Publication or Category) from current explorer Category
   * @name Explorer#removeItemFromCategory
   * @param {String} itemId
   * @function
   */
  Explorer.prototype.removeItemFromCategory = function (itemId) {
    var $activeItems = $('.explorer-item.active');
    var activeItemIdData = $activeItems.exists() ? ExplorerUtils.getItemIds($activeItems) : itemId;
  
    const currentExplorer = this;
    $.ajax({
      url: ACTION_URL,
      explorer: currentExplorer,
      traditional: true,
      data: {
        items: activeItemIdData,
        opRemoveCategory: true,
        explorerCurrentCategory: EXPLORER.data.currentCategoryId
      },
      type: 'post'
    })
      .done(function (data) {
        data.data.removedItemIds.forEach(function (itemId) {
          EXPLORER.removeItemById(itemId);
        });
        ExplorerUtils.displayMessage(data.message, data.actionStatus === 'ERROR');
      })
      .fail(function () {
        //TODO handle fail (Server error)
      });
  };

  /**
   * Adds item to selection (Mark the item as active, and check the checkbox)
   * @name Explorer#selectItem
   * @param {jQuery} $element
   * @function
   */
  Explorer.prototype.isUnclassifiedFilterEnabled = function () {
    return $(".is-uncategorized-access").hasClass("active");
  }
  
    /**
   * Focus breadcrumb after accessing an item (Folder) for A11y
   * @name Explorer#focusBreadcrumb
   * @function
   */
  Explorer.prototype.focusBreadcrumb = function () {
    this.$element.find(".explorer-breadcrumb-wrapper").addClass("js-focused");
    this.$element.find(".explorer-breadcrumb-wrapper")[0].focus({
      focusVisible: false
    });
  }

  /**
 * Update right icons on folders
 * @name Explorer#updateRightIcons
 * @param {String} categoryId the id of the category
 * @param {jQuery} categoryIconSrc the new src of the icon
 * @function
 */
  Explorer.prototype.updateRightIcons = function (categoryId, categoryIconSrc) {
    //TODO : update icon based on real css class
    var $updatedIcons = $("[data-jalios-data-id=" + categoryId + "] .js-icon");
    $updatedIcons.each(function (idx, element) {
      let $element = $(element);
      $element.removeClass("jcmsplugin-explorer-folder-rights").removeClass("icon-folder-rights");
      $element.removeClass("jcmsplugin-explorer-folder").removeClass("icon-folder");
      if (categoryIconSrc === "jcmsplugin-explorer-folder-rights") {
        $element.addClass("icon-folder-rights");
      } else {
        $element.addClass("icon-folder");
      }
      $element.addClass(categoryIconSrc);
    });
    // Update publication state cell
    $("[data-jalios-data-id=" +  categoryId + "] .state-wrapper").refresh({
      nohistory: true,
      params: {
        explorerCurrentCategory: EXPLORER.data.currentCategoryId,
        portletExplorer: EXPLORER.data.portletExplorerId,
        explorerListCategory: categoryId
      }
    });
    
  }

  /**
 * Refreshes breadcrumb
 * @name Explorer#updateBreadcrumb
 * @function
 */
  Explorer.prototype.updateBreadcrumb = function () {
    $(".explorer-breadcrumb-wrapper").refresh({
      nohistory: true,
      params: ExplorerUtils.getExplorerFormDataAsJsonObject(true),
      callback: function () {
      }
    });
  }
  
  Explorer.prototype.removeAllSelectedItems = function() {
    // Remove active class TODO : add method for this
    $(".explorer-item", this.$element).removeClass("active");
    $(".checkbox-check", this.$element).prop("checked", false);
    ExplorerUtils.removeCategoryItemsFromSelection();
    ExplorerUtils.removePublicationItemsFromSelection();
  }

  /**
   * Adds item to selection (Mark the item as active, and check the checkbox)
   * @name Explorer#selectItem
   * @param {jQuery} $element
   * @function
   */
  Explorer.prototype.selectItem = function ($item, isCtrlKeyPressed, isShiftKeyPressed, itemId) {
    const currentExplorer = $.plugin.ExplorerApp.getExplorer($item);
    $allContextMenus.hide();
    if (!itemId) {
      itemId = $item.data('jalios-data-id');
    }
    var $thisItem = $item;
    if (isShiftKeyPressed && !$item.hasClass('sidebar-categorytree-item')) {
      var $itItem = $item;
      var $startingItem = $lastActiveItem;
      var $explorerItems = $item.closest('.explorer-item-wrapper').find('.explorer-item');
      var triggerIndex = $explorerItems.index($item.get(0));
      var activeItemIndex = $explorerItems.index($lastActiveItem.get(0));
      var isBefore = triggerIndex < activeItemIndex;
      consoleDebugGroup('Multi selection through shift key', [
        'Active item index : ' + activeItemIndex,
        'Is before : ' + isBefore,
        'Trigger index ' + triggerIndex
      ]);
      if (isBefore) {
        while ($itItem.length > 0 && $itItem.data('jalios-data-id') !== $startingItem.data('jalios-data-id')) {
          ExplorerUtils.addItemToSelection($itItem, false, isCtrlKeyPressed, isShiftKeyPressed);
          if ($itItem.hasClass('card-publication') || $itItem.hasClass('explorer-row-item')) {
            // Get card-link, next card-link and retrieve item
            $itItem = $itItem.parent().next('.card-link').children();
          } else {
            //Category item
            $itItem = $itItem.next('.explorer-item');
          }
        }
      } else {
        while ($itItem.length > 0 && $itItem.data('jalios-data-id') !== $startingItem.data('jalios-data-id')) {
          ExplorerUtils.addItemToSelection($itItem, false, isCtrlKeyPressed, isShiftKeyPressed);
          if ($itItem.hasClass('card-publication')) {
            // Get card-link, next card-link and retrieve item
            $itItem = $itItem.parent().prev('.card-link').children();
          } else {
            //Category item
            $itItem = $itItem.prev('.explorer-item');
          }
        }
      }
    } else if (isCtrlKeyPressed && !$item.hasClass('sidebar-categorytree-item')) {
      ExplorerUtils.addItemToSelection($item, false, isCtrlKeyPressed, isShiftKeyPressed);
    } else {
      if ($item.hasClass('sidebar-categorytree-item')) {
        ExplorerUtils.toggleExplorerLoading();
        $('.sidebar-categorytree-item').removeClass('active').find("> A").removeAttr("aria-current");
        $item.addClass('active').find("> A").attr("aria-current", "true");
        $('.explorer-search-input').val('');

        let formData = ExplorerUtils.getExplorerFormData(false);
        formData.append('explorerCurrentCategory', $item.data('jalios-data-id'));

        if ($item.hasClass('is-uncategorized-access')) {
          formData.append('showUncategorizedContent', true);
        }

        var stateObj = {
          explorerCurrentCategory: $item.data('jalios-data-id')
        };

        let historyUrl = $thisItem.find('> A').data('jalios-explorer-history');
        if (historyUrl && historyUrl.startsWith('?')) {
          historyUrl =
            location.protocol +
            '//' +
            location.host +
            location.pathname +
            $thisItem.find('> A').data('jalios-explorer-history');
        }

        window.history.pushState(stateObj, $item.find('.js-title').text() + ' - ' + initialPageTitle, historyUrl);
        document.title = $item.find('.js-title').text() + ' - ' + initialPageTitle ;

        $('.app-body').refresh({
          nohistory: true,
          explorer: currentExplorer,
          params: ExplorerUtils.getFormDataAsJsonObject(formData),
          callback: function () {
            EXPLORER.data.currentCategoryId = $item.data('jalios-data-id');
            EXPLORER.refreshNoResultDisplay();
            ExplorerUtils.hideExplorerLoading();
            ExplorerUtils.clearSelection();
            EXPLORER.focusBreadcrumb();
            ExplorerUtils.handleForbiddenState();
            $.plugin.ExplorerApp.triggerEvent('movedToCategory', {
              categoryId: $item.data('jalios-data-id')
            });
          }
        });
      } else if ($item.hasClass('explorer-item-data')) {
        $explorerItemData = $item;
        sidebarPublicationId = itemId;
        EXPLORER_PREVIEW_SIDEBAR.open({
          "sidebarTab": sidebarTab
        });
      } else {
        $('.sidebar-categorytree-item').removeClass('active');
        // Sidebar category root : mark as active
        if ($item.hasClass('is-root-access')) {
          $('.sidebar-categorytree-item.is-root-access').addClass('active');
        } else if ($item.data('jalios-data-id')) {
          $('.sidebar-categorytree-item[data-jalios-data-id=' + $item.data('jalios-data-id') + ']').addClass('active');
        }

        // Explorer Category
        ExplorerUtils.toggleExplorerLoading();

        var stateObj = {
          explorerCurrentCategory: $item.data('jalios-data-id')
        };

        let historyUrl = $thisItem.data('jalios-explorer-history');
        if (historyUrl && historyUrl.startsWith('?')) {
          historyUrl =
            location.protocol + '//' + location.host + location.pathname + $thisItem.data('jalios-explorer-history');
        }

        window.history.pushState(stateObj, $item.find('.js-title').text() + ' - ' +  initialPageTitle , historyUrl);

        $('.explorer-search-input').val('');
        let formData = ExplorerUtils.getExplorerFormData(false);
        formData.append('explorerCurrentCategory', $item.data('jalios-data-id'));
        document.title = $item.find('.js-title').text() + ' - ' + initialPageTitle ;

        let scrollToPreviousElementId = EXPLORER.data.currentCategoryId;
        let isScrollToFolderEnabled = $('.app-body').data('jalios-explorer-app-scroll-to-previous-folder');
        $('.app-body').refresh({
          nohistory: true,
          noscroll: isScrollToFolderEnabled,
          explorer: currentExplorer,
          params: ExplorerUtils.getFormDataAsJsonObject(formData),
          callback: function () {
            EXPLORER.refreshNoResultDisplay();
            EXPLORER.data.currentCategoryId = $item.data('jalios-data-id');
            EXPLORER.focusBreadcrumb();
            ExplorerUtils.clearSelection();
            ExplorerUtils.hideExplorerLoading();
            $.plugin.ExplorerApp.triggerEvent('movedToCategory', {
              categoryId: $item.data('jalios-data-id')
            });
            ExplorerUtils.handleForbiddenState();

            if (isScrollToFolderEnabled && scrollToPreviousElementId) {
              let $scrollToPreviousElement = $('.explorer-item[data-jalios-data-id=' + scrollToPreviousElementId + ']');
              if ($scrollToPreviousElement.exists()) {
                // Scroll to top if element position is under 600
                let elementTop = $scrollToPreviousElement.offset().top;
                $([document.documentElement, document.body]).animate(
                  {
                    scrollTop: elementTop < 600 ? 0 : elementTop - 104
                  },
                  0
                );
              } else {
                let elementTop = currentExplorer.$element.offset().top;
                $([document.documentElement, document.body]).animate(
                  {
                    scrollTop: elementTop < 600 ? 0 : elementTop - 104
                  },
                  0
                );
              }
            }
          }
        });
      }
    }
  };

  // ------------------------------------------
  //  Sidebar handling
  // ------------------------------------------

  const initSidebar = function () {
    // EXPLORER SIDEBAR : displays a publication

    $('#explorer-preview-sidebar').sidebar();
    EXPLORER_PREVIEW_SIDEBAR = $('#explorer-preview-sidebar').data('sidebar');

    // Lazy load items in the sidebar
    $(document).on('show.jalios.sidebar', function (event) {
      $allContextMenus.css('display', 'none');

      var $explorerSidebar = $(event.target);
      if (!$explorerSidebar.hasClass('explorer-preview-sidebar')) {
        return;
      }

      var title = $explorerItemData.find('.js-title').text();
      var icon = $explorerItemData.find('.js-title .jalios-icon').clone();
      icon.removeAttr("aria-hidden");

      $('.data-title .data-icon').html('');
      $('.data-title .data-icon').append(icon);
      $('.data-title .data-label').text(title);

      var currentDataId = $explorerItemData.data('jalios-data-id');
      $explorerSidebar.data('jalios-data-id', currentDataId);

      let publicationUrl = new URL(document.location);
      publicationUrl.searchParams.set("displayedPublication", currentDataId);

      $explorerSidebar.data("previousUrl", document.location);
      window.history.pushState(null, initialPageTitle + ' - ' + title, publicationUrl);
      document.title = initialPageTitle + ' - ' + title;

      $('.explorer-preview-sidebar .sidebar-body').refresh({
        noscroll: true,
        nohistory: true,
        params: {
          data: currentDataId,
          portletExplorer: EXPLORER.data.portletExplorerId,
          explorerUrl: $('.explorer-current-url').val(),
          explorerCurrentCategory: EXPLORER.data.currentCategoryId,
          explorerSearchPerformed: $('.explorer-search-result-displayed').val(),
          sidebarTab: sidebarTab
        },
        callback: function () {
          $.ajax({
            url: ACTION_URL,
            traditional: true,
            data: {
              items: currentDataId,
              opIncrementReaderTracker: true
            },
            type: 'post'
          });
        }
      });

      $('.dropdown.open .dropdown-toggle').dropdown('toggle');
    });
    // EDIT SIDEBAR

    $('#explorer-edit-sidebar').sidebar();
    EXPLORER_EDIT_SIDEBAR = $('#explorer-edit-sidebar').data('sidebar');

    $(document).on('start-hide.jalios.sidebar', function (event) {
      var $explorerSidebar = $(event.target);
      if (!$explorerSidebar.hasClass('explorer-preview-sidebar')) {
        return;
      }


      let explorerUrl = new URL($explorerSidebar.data("previousUrl"));
      explorerUrl.searchParams.delete("displayedPublication");
      window.history.replaceState(null, initialPageTitle, explorerUrl);
      document.title = initialPageTitle;
    });

    // Lazy load items in the sidebar
    $(document).on('show.jalios.sidebar', function (event) {
      var $explorerSidebar = $(event.target);
      if (!$explorerSidebar.hasClass('explorer-edit-sidebar')) {
        return;
      }

      var $itemCard = $('[data-jalios-data-id=' + EXPLORER_CTX_MENU_ITEM_ID + ']');

      var title = $itemCard.find('.card-title').text();
      var icon = $itemCard.find('.card-title .jalios-icon').clone();

      $('.data-title .data-icon').html('');
      $('.data-title .data-icon').append(icon);
      $('.data-title .data-label').text(title);

      var currentDataId = $itemCard.data('jalios-data-id');
      $explorerSidebar.data('jalios-data-id', currentDataId);

      $('.explorer-edit-sidebar .sidebar-body').refresh({
        noscroll: true,
        nohistory: true,
        params: {
          dataId: currentDataId
        }
      });

      // var $lazyElement = $(".site-sidebar .sidebar-lazyload");
      // $lazyElement.each(function(index, element) {
      //   var $element = $(element);
      //   $element.displayLoadingWaves();
      //   var lazyUrl = $element.data("jalios-sidebar-lazyload-url");
      //   $.jalios.AjaxRefresh.refresh(lazyUrl, $element, {
      //     noscroll: true,
      //     nohistory: true
      //   });
      // });
    });
  };

  // ------------------------------------------
  //  Utils
  // ------------------------------------------

  const ExplorerUtils = {
    handleForbiddenState: function () {
      if ($('.app-main').hasClass('is-upload-forbidden') && !$('.app-body').hasClass('is-upload-forbidden')) {
        $('.app-main').removeClass('is-upload-forbidden');
      } else if (!$('.app-main').hasClass('is-upload-forbidden') && $('.app-body').hasClass('is-upload-forbidden')) {
        $('.app-main').addClass('is-upload-forbidden');
      }
    },
    handleTableDisplay: function () {
      if ($('.explorer-item-category').length == 0) {
        $('.explorer-table').addClass('hide');
        if (ExplorerUtils.getDisplayMode() === 'LIST') {
          $('.app-no-results').addClass('hide');
        }
      } else {
        $('.explorer-table').removeClass('hide');
        if (ExplorerUtils.getDisplayMode() === 'LIST') {
          $('.app-no-results').removeClass('hide');
        }
      }
    },
    getExplorerFormDataAsJsonObject: function (addCurrentCategory) {
      var formData = ExplorerUtils.getExplorerFormData(addCurrentCategory);
      return ExplorerUtils.getFormDataAsJsonObject(formData);
    },
    getExplorerFormData: function (addCurrentCategory) {
      var formData = new FormData($('.explorer-form').get(0));
      formData.append('portletExplorer', EXPLORER.data.portletExplorerId);
      formData.append('showUncategorizedContent', EXPLORER.isUnclassifiedFilterEnabled());
      if (addCurrentCategory) {
        formData.append('explorerCurrentCategory', EXPLORER.data.currentCategoryId);
      }
      formData.append('displayMode', ExplorerUtils.getDisplayMode());
      return formData;
    },
    getFormDataAsJsonObject: function (formData) {
      var object = {};
      formData.forEach(function (value, key) {
        if (object[key] && key === 'pinnedCategoryIds') {
          let arrayLength = Array.isArray(object[key]) ? object[key].length : 1;
          let multiValue = [];
          if (arrayLength > 1) {
            multiValue = object[key];
            multiValue = multiValue.concat(value);
          } else {
            multiValue[0] = object[key];
            multiValue[arrayLength] = value;
          }

          object[key] = multiValue;
        } else {
          object[key] = value;
        }
      });
      return object;
    },
    getDisplayMode: function () {
      return $('.js-change-display-mode:checked').val();
    },
    getOpenedSidebarCategories: function () {
      var openedCategories = [];
      $('.sidebar-categorytree-item.is-opened').each(function (index, element) {
        openedCategories.push($(element).data('jalios-data-id'));
      });
      return openedCategories;
    },
    clearSelection: function () {
      if (document.selection && document.selection.empty) {
        document.selection.empty();
      } else if (window.getSelection) {
        var sel = window.getSelection();
        sel.removeAllRanges();
      }
    },
    displayMessage: function (message, isError) {
      toastr.options = {
        positionClass: 'toast-top-center'
      };
      if (isError) {
        toastr.error(message);
      } else {
        toastr.info(message);
      }
    },
    showExplorerLoading: function () {
      if (!$('.app-body').find('.ajax-loading').exists()) {
        $('.app-body').displayLoading();
      }
    },
    hideExplorerLoading: function () {
      $('.app-body .ajax-loading').remove();
    },
    toggleExplorerLoading: function () {
      if ($('.app-body').find('.ajax-loading').exists()) {
        $('.app-body .ajax-loading').remove();
      } else {
        $('.app-body').displayLoading();
      }
    },
    getItemIds: function ($elements) {
      return $elements
        .map(function () {
          return $(this).data('jalios-data-id');
        })
        .get();
    },
    addItemToSelection: function ($element, handleCtrlKey, isCtrlKeyPressed, isShiftKeyPressed) {
      consoleDebug('Adding item ' + $element.data('jalios-data-id') + ' to selection');
      $lastActiveItem = $element;

      EXPLORER_CTX_MENU_ITEM_ID = $element.data('jalios-data-id');

      // Toggle select/unselect label
      if ($element.hasClass('active')) {
        $element.find('.js-activate-card').attr('data-original-title', $element.find(".js-activate-card").data("jalios-select-title"));
      } else {
        $element.find('.js-activate-card').attr('data-original-title', $element.find(".js-activate-card").data("jalios-unselect-title"));
      }

      if (isShiftKeyPressed) {
        $element.attr('aria-selected', true);
        $element.addClass('active');
      } else {
        if ($element.hasClass('active')) {
          $element.attr('aria-selected', false);
          $element.removeClass('active');
        } else {
          $element.attr('aria-selected', true);
          $element.addClass('active');
        }
      }

      // Only allow select of item of same type
      if ($element.hasClass('card-publication') || $element.hasClass('explorer-item-data')) {
        ExplorerUtils.removeCategoryItemsFromSelection();
      } else {
        //Category
        ExplorerUtils.removePublicationItemsFromSelection();
      }
      if ((isCtrlKeyPressed && !handleCtrlKey) || isShiftKeyPressed) {
        consoleDebugGroup('Check item checkbox', $element);
        $element
          .find('.checkbox-check')
          .prop('checked', isShiftKeyPressed ? true : !$element.find('.checkbox-check').prop('checked'));
      }
      const explorer = $.plugin.ExplorerApp.getExplorer($element);
      explorer.updateSelectedElementsNavbarItem();
      updateActionsMenu();
    },
    removePublicationItemsFromSelection: function () {
      $('.card-publication, .explorer-item-data').each(function (idx, element) {
        let $item = $(element);

        // Toggle select/unselect label
        if ($item.hasClass('active')) {
          $item.find('.js-activate-card').attr('data-original-title', explorerAppI18Nprops.selectLabel);
        }
        $item.attr('aria-selected', false);
        $item.removeClass('active');

        // Uncheck checkbox
        $item.find('.checkbox-check').prop('checked', false);
      });
    },
    removeCategoryItemsFromSelection: function () {
      $('.card-folder, .explorer-item-category').each(function (idx, element) {
        let $item = $(element);

        // Toggle select/unselect label
        if ($item.hasClass('active')) {
          $item.find('.js-activate-card').attr('data-original-title', explorerAppI18Nprops.selectLabel);
        }

        $item.removeClass('active');
        $item.attr('aria-selected', false);

        // Uncheck checkbox
        $item.find('.checkbox-check').prop('checked', false);
      });
    }
  };

  const onKeyUp = function (event) {
    IS_CTRL_KEY_PRESSED = false;
    IS_SHIFT_KEY_PRESSED = false;
  };

  const onKeyDown = function (event) {
    if (event.which === CTRL_KEY) {
      IS_CTRL_KEY_PRESSED = true;
    }
    if (event.which === SHIFT_KEY) {
      IS_SHIFT_KEY_PRESSED = true;
    }
  };

  const updateActionsMenu = function ($activeItem) {
    // If an item has been passed, use it instead of getting selected items (active)
    let $activeItems = $($activeItem).exists() ? $($activeItem) : $('.explorer-item.active');
    let activeItemIds = ExplorerUtils.getItemIds($activeItems);

    if (activeItemIds.length === 0) {
      if ($('.explorer-actions-menu').hasClass('has-no-items') && activeItemIds.length === 0) {
        return;
      }
      $('.explorer-actions-menu').addClass('has-no-items');
    } else {
      $('.explorer-actions-menu').removeClass('has-no-items');
    }

    const currentExplorer = $.plugin.ExplorerApp.getExplorer($activeItem);

    $.ajax({
      url: CTXT_MENU_URL,
      traditional: true,
      explorer: currentExplorer,
      data: {
        contextMenuDataId: activeItemIds,
        multipleItems: $activeItems.length > 1,
        explorerCurrentCategory: EXPLORER.data.currentCategoryId,
        portletExplorer: EXPLORER.data.portletExplorerId,
        originalExplorerUrl: EXPLORER.$element.find('input[name=originalExplorerUrl]').val(),
        isExplorerSearch: $('.explorer-search-result-displayed').val() === 'true' ? true : false
      },
      type: 'post'
    }).done(function (html) {
      $('.explorer-actions-menu').html(html);
    });
  };



  const updateCardContextMenu = function ($item, event, forceUniqueItem) {
    const currentExplorer = $.plugin.ExplorerApp.getExplorer($item);

    if ($item.hasClass('is-root-access')) {
      return;
    }

    // Check there's multiple active card (Multi selection)
    var $activeItems = $('.explorer-item.active');
    if ($activeItems.length > 1 && !forceUniqueItem) {
      var activeItemIds = ExplorerUtils.getItemIds($activeItems);

      $.ajax({
        url: CTXT_MENU_URL,
        traditional: true,
        data: {
          contextMenuDataId: activeItemIds,
          multipleItems: true,
          explorerCurrentCategory: currentExplorer.data.currentCategoryId,
          originalExplorerUrl: currentExplorer.$element.find('input[name=originalExplorerUrl]').val(),
          portletExplorer: currentExplorer.data.portletExplorerId,
          isExplorerSearch: $('.explorer-search-result-displayed').val() === 'true' ? true : false,
          isBreadcrumbItem: $item.closest('.breadcrumb-item').exists(),
          isSidebarCategoryNode: $item.closest('.sidebar-categorytree-item').exists()
        },
        explorer: currentExplorer,
        type: 'post'
      }).done(function (html) {
        $('#explorerContextMenu .dropdown-menu').html(html);
        $('#explorerContextMenu').css({
          display: 'block',
          left: event.pageX,
          top: event.pageY - document.documentElement.scrollTop
        });
        $('#explorerContextMenu > UL > LI:first > A').addClass("js-focused");
        if ($('#explorerContextMenu > UL > LI:first > A').exists()) {
          $('#explorerContextMenu > UL > LI:first > A')[0].focus({
            focusVisible: false
          });
        }
        // Clean up menu dividers
        $('#explorerContextMenu .dropdown-menu .divider:first-child').remove();
        $('#explorerContextMenu .dropdown-menu .divider:last-child').remove();
        $('#explorerContextMenu .dropdown-menu .divider + .divider').remove();
        $('#explorerContextMenu .dropdown-menu LI:not(.divider):empty').remove();
      });

      return;
    }

    if ($item.exists()) {
      EXPLORER_CTX_MENU_ITEM_ID = $item.data('jalios-data-id');
    }

    $.ajax({
      url: CTXT_MENU_URL,
      explorer: currentExplorer,
      data: {
        contextMenuDataId: $item ? $item.data('jalios-data-id') : null,
        explorerCurrentCategory: currentExplorer.data.currentCategoryId,
        portletExplorer: currentExplorer.data.portletExplorerId,
        originalExplorerUrl: currentExplorer.$element.find('input[name=originalExplorerUrl]').val(),
        isExplorerSearch: $('.explorer-search-result-displayed').val() === 'true' ? true : false,
        isSidebarCategoryNode: $item.closest('.sidebar-categorytree-item').exists(),
        isBreadcrumbItem: $item.closest('.breadcrumb-item').exists()
      },
      type: 'post'
    }).done(function (html) {
      $('#explorerContextMenu .dropdown-menu').html(html);
      $('#explorerContextMenu').css({
        display: 'block',
        top: 0,
        left: 0,
        right: '',
        bottom: ''
      });

      // Clean up menu dividers
      $('#explorerContextMenu .dropdown-menu .divider:first-child').remove();
      $('#explorerContextMenu .dropdown-menu .divider:last-child').remove();
      $('#explorerContextMenu .dropdown-menu .divider + .divider').remove();

      var element = $('#explorerContextMenu').get(0);
      var isInViewPortHorizontal = elementInViewportHorizontal(element, event.pageX);
      var isInViewPortVertical = elementInViewportVertical(element, event.pageY);

      $('#explorerContextMenu').css({
        right: !isInViewPortHorizontal ? 0 : '',
        left: !isInViewPortHorizontal ? '' : event.pageX,
        top: !isInViewPortVertical ? '' : event.pageY - document.documentElement.scrollTop,
        bottom: !isInViewPortVertical ? 0 : ''
      });
      $('#explorerContextMenu > UL > LI:first > A').addClass("js-focused");
      $('#explorerContextMenu > UL > LI:first > A')[0].focus({
        focusVisible: false
      });
    });
  };

  // ------------------------------------------
  //  Drag & Drop
  // ------------------------------------------
  const elementInViewportHorizontal = function (el, xPosition) {
    return xPosition + el.offsetWidth < window.innerWidth;
  };

  const elementInViewportVertical = function (el, yPosition) {
    return yPosition - document.documentElement.scrollTop + el.offsetHeight < window.innerHeight;
  };

  const initDraggableAndDroppable = function (explorer) {
    if (explorer.isDragAndDropAvailable) {
      initDraggable();
      initDroppable();
    }
  };

  const initEventHandling = function (explorer) {
    explorer.$element.on('jalios.explorer', function (event) {
      switch (event.explorerEvent) {
        case "updateCategoryRights":
          EXPLORER.updateRightIcons(event.explorerParams.categoryId, event.explorerParams.categoryIconSrc);
          EXPLORER.updateBreadcrumb();
          break;
        case 'newCategory':
          EXPLORER.refreshCategoryList(event.explorerParams.categoryCreatedIds);

          var openedCategories = ExplorerUtils.getOpenedSidebarCategories();
          if (event.explorerParams.openTreeNode) {
            openedCategories.push(event.explorerParams.openTreeNode);
          }

          $('.app-sidebar-categorytree').refresh({
            nohistory: true,
            params: {
              explorerCurrentCategory: EXPLORER.data.currentCategoryId,
              portletExplorer: EXPLORER.data.portletExplorerId,
              sidebarOpenedCategories: openedCategories
            },
            callback: function () {
              EXPLORER.refreshNoResultDisplay();
            }
          });

          break;
        case 'mergeCategory':
          ExplorerUtils.displayMessage(event.explorerParams.message, false);
          EXPLORER.removeItemById(event.explorerParams.mergedCategoryId);
          break;
        case 'rightsUpdated':
          $('[data-jalios-data-id=' + event.explorerParams.dataId + '] .state-wrapper').refresh({
            nohistory: true,
            params: {
              explorerListPub: event.explorerParams.dataId,
              explorerCurrentCategory: EXPLORER.data.currentCategoryId,
              portletExplorer: EXPLORER.data.portletExplorerId,
            }
          });
          ExplorerUtils.displayMessage(event.explorerParams.message, false);
          initTooltips();
          break;
      }
    });
  };

  const handleDropAction = function (dropTargetId, $droppedItem) {
    if ($droppedItem.length === 0 || !dropTargetId) {
      // TODO : toastr for error
      return;
    }
    const currentExplorer = $.plugin.ExplorerApp.getExplorer($droppedItem);

    var $activeItems = $('.explorer-item.active');
    var activeItemIdData =
      $activeItems.length > 1 ? ExplorerUtils.getItemIds($activeItems) : $draggedItem.data('jalios-data-id');

    $.ajax({
      url: ACTION_URL,
      traditional: true,
      explorer: currentExplorer,
      data: {
        items: activeItemIdData,
        portletExplorer: EXPLORER.data.portletExplorerId,
        explorerCurrentCategory: EXPLORER.data.currentCategoryId,
        targetCategory: dropTargetId,
        unclassified: EXPLORER.isUnclassifiedFilterEnabled(),
        opMove: true
      },
      type: 'post'
    })
      .done(function (data) {
        if (data.actionStatus === 'ERROR') {
          ExplorerUtils.displayMessage(data.message, true);
          return;
        }

        if (data.data && data.data.movedItemIds && data.data.movedItemIds.length === 0) {
          return;
        }

        if ($(".is-root-access.sidebar-categorytree-item[data-jalios-data-id=" + dropTargetId + "]").exists()) {
          $.plugin.ExplorerApp.refreshFullCategoryListInSidebar();
        } else if (EXPLORER.data.currentCategoryId === dropTargetId) {
          // Refresh category list in body
          $.plugin.ExplorerApp.refreshCategoryList();
        } else {
          $.plugin.ExplorerApp.refreshCategoryNode(dropTargetId, true);
        }

        if (data.data && data.data.movedItemIds) {
          data.data.movedItemIds.forEach(function (itemId) {
            $('.sidebar-categorytree-item[data-jalios-data-id=' + itemId + ']').remove();
            EXPLORER.removeItemById(itemId);
          });
        }

        ExplorerUtils.displayMessage(data.message, data.actionStatus !== 'OK');
      })
      .fail(function () {
        //TODO handle fail (Server error)
      });
  };

  const initDroppable = function (selector) {
    var dropSelector = selector
      ? selector
      : '.js-draggable-elements .explorer-item-folder, .sidebar-categorytree-item:not(.is-root-access) > A, .js-draggable-elements .explorer-item-category, .breadcrumb-item-wrapper:not(:last-child) .breadcrumb-item';
    $(dropSelector).droppable({
      //activeClass: "ui-state-highlight",
      // accept: ".explorer-item-folder",
      greedy: true,
      over: function (event, ui) {
        $('.explorer-move-tooltip').addClass('is-displayed');
        $('.explorer-move-tooltip-category').text($(this).find('.js-title').text());

        let $srcElement =
          event.originalEvent && event.originalEvent.originalEvent && $(event.originalEvent.originalEvent.srcElement);
        let isElementManageable = $('.explorer-helper').hasClass('is-manageable');
        let isTargetManageable = $(event.target).closest('.sidebar-categorytree-item').exists()
          ? $(event.target).closest('.sidebar-categorytree-item').hasClass('is-manageable')
          : $(event.target).hasClass('is-manageable');
        let isCategoryDragging = $('.explorer-helper').hasClass('explorer-item-category');

        if (isCategoryDragging && (!isTargetManageable || !isElementManageable)) {
          $(this).addClass('is-drag-hover-forbidden');
          $('.explorer-move-tooltip').addClass('is-forbidden');
        } else if (isTargetManageable && isElementManageable) {
          $(this).addClass('is-drag-hover');
        } else if ($srcElement && $srcElement.closest('.explorer-item').hasClass('is-uneditable')) {
          $(this).addClass('is-drag-hover-forbidden');
          $('.explorer-move-tooltip').addClass('is-forbidden');
        } else if ($(this).hasClass('is-usable')) {
          $(this).addClass('is-drag-hover');
        } else {
          $(this).addClass('is-drag-hover-forbidden');
          $('.explorer-move-tooltip').addClass('is-forbidden');
        }
      },
      drop: function (event, ui) {
        var $activeItems = $('.explorer-item.active');
        $(this).removeClass('is-drag-hover');
        $(this).removeClass('is-drag-hover-forbidden');
        $('.explorer-move-tooltip').removeClass('is-forbidden');
        if ($activeItems.length > 1) {
          handleDropAction($(event.target).data('jalios-data-id'), $activeItems);
        } else {
          handleDropAction($(event.target).data('jalios-data-id'), $draggedItem);
        }
        consoleDebug(['Moving' + $draggedItem + ' to : ' + event.target, $draggedItem, event.target]);
        $('.explorer-move-tooltip').removeClass('is-displayed');
      },
      out: function (event, ui) {
        $(this).removeClass('is-drag-hover');
        $(this).removeClass('is-drag-hover-forbidden');
        $('.explorer-move-tooltip').removeClass('is-forbidden');
        $('.explorer-move-tooltip').removeClass('is-displayed');
      },
      // drop: onBlockDroppedHandler,
      tolerance: 'pointer'
    });
  };

  const initDraggable = function (selector) {
    var dragSelector = selector
      ? selector
      : '.js-draggable-elements .explorer-item, .sidebar-categorytree-item.is-usable, .sidebar-categorytree-item.is-manageable';

    $(dragSelector).draggable({
      start: function (event, ui) {
        $('.app-explorer').addClass('is-dragndropping');

        $draggedItem = $(event.currentTarget);
        let $chidrenDraggedItem = $(event.originalEvent.target).closest('.sidebar-categorytree-item');

        if ($draggedItem.hasClass('sidebar-categorytree-item') && $chidrenDraggedItem.exists()) {
          $draggedItem = $chidrenDraggedItem;
        }

        var $activeItems = $('.explorer-item.active');
        if ($activeItems.length > 1) {
          $('.explorer-helper').html("<span class='badge badge-default'>" + $activeItems.length + '</span> Items');
          $activeItems.addClass('is-dragged');
          if ($($draggedItem.get(0)).hasClass('explorer-item-category')) {
            $('.explorer-helper').addClass('explorer-item-category');
          }
          // Check all items are manageable
          if ($activeItems.filter('.is-manageable').length === $activeItems.length) {
            $('.explorer-helper').addClass('is-manageable');
          }
        } else {
          if ($draggedItem.hasClass('is-manageable')) {
            $('.explorer-helper').addClass('is-manageable');
          }
          if ($draggedItem.hasClass('explorer-item-category')) {
            $('.explorer-helper').addClass('explorer-item-category');
          }
          $('.explorer-helper').text($(event.currentTarget).find('.js-title').text());
          var icon = $(event.currentTarget).find('.js-icon').first().clone();
          $('.explorer-helper').prepend(icon);
          $draggedItem.addClass('is-dragged');
        }
      },
      drag: function (e, ui) { },
      stop: function (event, ui) {
        $('.is-dragged').removeClass('is-dragged');
        $draggedItem = undefined;
        $('.app-explorer').removeClass('is-dragndropping');
        $('.explorer-helper').removeClass('is-manageable');
        $('.explorer-helper').removeClass('explorer-item-category');
      },
      containment: 'body',
      scrollSensitivity: 100,
      scrollSpeed: 30,
      delay: 150,
      appendTo: '.app-body',
      helper: function () {
        return "<div class='explorer-helper'>/div>";
      },
      cursorAt: {
        left: 5,
        top: 0
      },
      cursor: 'drag'
    });
  };

  const consoleDebugGroup = function (title, logs) {
    consoleDebug(logs, title);
  };

  const initTooltips = function () {
    $('[data-toggle="tooltip"]').tooltip({
      container: 'body'
    });
  };

  const consoleDebug = function (logs, title) {
    if (EXPLORER && EXPLORER.debugMode) {
      var isArray = Array.isArray(logs);
      var isDebugEnabled = true;
      if (isArray) {
        console.group('[Explorer] ' + title);
        logs.forEach(function (element) {
          $.console.log(element);
        });
        console.groupEnd();
      } else {
        if (typeof logs === 'string') {
          $.console.log('[Explorer] ' + logs);
        } else {
          console.group('[Explorer] ' + title);
          $.console.log(logs);
          console.groupEnd();
        }
      }
    } else if (!EXPLORER) {
      $.console.log(logs);
    }
  };
  // ------------------------------------------
  //  PUBLIC CODE
  // ------------------------------------------

  // Namespace
  if (!$.plugin) {
    $.plugin = {};
  }

  /**
   * Documentation of the ExplorerApp public API.<br/>
   *
   * @namespace $.plugin.ExplorerApp
   */
  $.plugin.ExplorerApp = {
    /**
     * Displays a toastr message to the user
     * @name $.plugin.ExplorerApp.displayMessage
     * @param {string} message the message to display
     * @param {boolean} isError true if the message displays an error
     * @function
     */
    displayMessage: function (message, isError) {
      ExplorerUtils.displayMessage(message, isError);
    },

    /**
     * Ajax-refresh on the category list of Explorer
     * @name $.plugin.ExplorerApp.refreshCategoryList
     * @param {string} categoryCreatedIds[] - The ids of created categories (optional)
     * @function
     */
    refreshFullCategoryListInSidebar: function () {
      EXPLORER.refreshFullCategoryListInSidebar();
    },

    /**
     * Ajax-refresh on the category list of Explorer
     * @name $.plugin.ExplorerApp.refreshCategoryList
     * @param {string} categoryCreatedIds[] - The ids of created categories (optional)
     * @function
     */
    refreshCategoryList: function (categoryCreatedIds) {
      EXPLORER.refreshCategoryList(categoryCreatedIds);
    },

    /**
     * Refresh category node in sidebar if subMenu is already opened
     * @name $.plugin.ExplorerApp.refreshCategoryNode
     * @function
     */
    refreshCategoryNode: function (categoryId, refreshParent) {
      var $node = $('.sidebar-categorytree-item[data-jalios-data-id=' + categoryId + ']');
      var $subMenu = $node.find('> .app-sidebar-menu');
      const currentExplorer = $.plugin.ExplorerApp.getExplorer($node);

      if ($subMenu.exists()) {
        $.ajax({
          url: CATEGORY_NODE_CHILDRENS_URL,
          explorer: currentExplorer,
          data: {
            portletExplorer: EXPLORER.data.portletExplorerId,
            explorerCurrentCategory: EXPLORER.data.currentCategoryId,
            openedCategoryId: categoryId
          },
          type: 'post'
        }).done(function (html) {
          $subMenu.replaceWith(html);
          initDroppable('.sidebar-categorytree-item:not(.is-root-access) > A');
        });
      } else if (refreshParent) {
        let parentCategoryId = $node.parent().closest('.sidebar-categorytree-item').data('jalios-data-id');
        $.plugin.ExplorerApp.refreshCategoryNode(parentCategoryId);
      }
    },
    /**
     * Triggers an event with given params
     * @name $.plugin.ExplorerApp.triggerEvent
     * @function
     */
    triggerEvent: function (eventName, params) {
      var event = jQuery.Event('jalios.explorer');
      if (params) {
        var paramsObj = typeof params === 'object' ? params : JSON.parse(params);
        event.explorerParams = paramsObj;
      }
      event.explorerEvent = eventName;
      $.plugin.ExplorerApp.getExplorer().$element.trigger(event);
    },
    
        /**
     * Returns the EXPLORER object
     * @name $.plugin.ExplorerApp.getExplorer
     * @param {Element} element - A children element to retrieve explorer
     * @function
     * @returns Explorer
     */
    getExplorer: function (element) {
      if (element) {
        if ($(element).closest(".app-explorer").exists()) {
          return $(element).closest(".app-explorer").data("explorer");
        } else {
          return EXPLORER;
        }
      }
      return EXPLORER;
    },
    
    /**
     * @name $.plugin.ExplorerApp.scrollToExplorer
     * @param {String} explorerId - scroll to given explorer Id (data id)
     * @function
     */
    scrollToExplorer: function(explorerId) {
      const $explorer = $(".app-explorer[data-jalios-explorer-portlet-id=" + explorerId + "]");
      if ($explorer.exists()) {
        $explorer[0].scrollIntoView();
      }
    },
    
    /**
     * Returns the explorer form data as a json object
     * @name $.plugin.ExplorerApp.getExplorerFormDataAsJsonObject
     * @function
     * @returns the explorer form data as json
     */
    getExplorerFormDataAsJsonObject: function (addCurrentCategory) {
      return ExplorerUtils.getExplorerFormDataAsJsonObject(addCurrentCategory);
    },
    /**
     * Returns the explorer FormData
     * @name $.plugin.ExplorerApp.getExplorerFormData
     * @function
     * @returns FormData
     */
    getExplorerFormData: function (addCurrentCategory) {
      return ExplorerUtils.getExplorerFormData(addCurrentCategory);
    },
    /**
     * Returns a converted formData as json object
     * @name $.plugin.ExplorerApp.getFormDataAsJsonObject
     * @param {FormData} formData - A FormData object
     * @function
     * @returns form data as json
     */
    getFormDataAsJsonObject: function (formData) {
      return ExplorerUtils.getFormDataAsJsonObject(formData);
    },
    /**
     * Hides explorer loading
     * @name $.plugin.ExplorerApp.hideExplorerLoading
     * @function
     */
    hideExplorerLoading: function () {
      ExplorerUtils.hideExplorerLoading();
    },
    /**
     * Toggles explorer loading
     * @name $.plugin.ExplorerApp.toggleExplorerLoading
     * @function
     */
    toggleExplorerLoading: function () {
      ExplorerUtils.toggleExplorerLoading();
    },
    /**
     * Adds a console debug to the browser
     * @name $.plugin.ExplorerApp.consoleDebug
     * @function
     */
    consoleDebug: function (logs, title) {
      consoleDebug(logs, title);
    },
    /**
     * Adds a console debug group to the browser
     * @name $.plugin.ExplorerApp.consoleDebugGroup
     * @function
     */
    consoleDebugGroup: function (title, logs) {
      consoleDebugGroup(title, logs);
    },
    /**
     * Returns the current explorer category name
     * @name $.plugin.ExplorerApp.getCurrentCategoryName
     * @function
     * @returns the current explorer category name
     */
    getCurrentCategoryName: function () {
      return $('.app-body').data('jalios-explorer-current-category-name');
    },
    /**
     * Returns true if current category is the root category of the explorer
     * @name $.plugin.ExplorerApp.isCurrentCategoryRoot
     * @function
     * @returns true if current category is the root category of the explorer
     */
    isCurrentCategoryRoot: function () {
      return $('.app-body').data('jalios-explorer-current-category-is-root');
    }
  };

  // ------------------------------------------
  //  Initialization
  // ------------------------------------------

  const initCtrlKeyTrigger = function () {
    $(document).keydown(onKeyDown);
    $(document).keyup(onKeyUp);
  };

  const initDisplayedPublication = function () {
    let $displayedPublicationInput = $(".displayed-publication");
    if ($displayedPublicationInput.exists()) {
      $explorerItemData = $displayedPublicationInput;
      sidebarPublicationId = $displayedPublicationInput.val();
      EXPLORER_PREVIEW_SIDEBAR.open({});
    }
  }

  const handleBodyRefresh = function (event) {
    if (event.refresh.type === 'before') {
      if (event.refresh.options && event.refresh.options.params) {
        event.refresh.options.params.showUncategorizedContent = EXPLORER.isUnclassifiedFilterEnabled();
        event.refresh.options.params.collaborativeSpaceApplicationDisplay = EXPLORER.isCollaborativeSpaceApplicationDisplay();
      }
      if ($(event.refresh.target).hasClass('explorer-results')) {
        ExplorerUtils.showExplorerLoading();
      }
      return;
    } else if (event.refresh.type !== 'after') {
      return;
    }

    if ($(event.refresh.target).hasClass('explorer-results')) {
      ExplorerUtils.hideExplorerLoading();
    }

    initTooltips();
    EXPLORER.refreshNoResultDisplay();
    EXPLORER.initAfterAjax();
  };

  $(document).ready(function () {
    initExplorer();
    initCtrlKeyTrigger();
    $(document).on('jalios:refresh', handleBodyRefresh);
    initTooltips();

    // Add active tab to rights modal to keep the same active tab 
    $(document).on("jalios:refresh", function(event) {
      if (event.refresh.type === "before" && $(event.refresh.target).find(".explorer-modal-edit-category-rights").exists()) {
        if ($(event.refresh.target).find(".explorer-modal-edit-category-rights").closest(".modal.in").exists()) {
          event.refresh.options.params.activeTab = $(event.refresh.target).find(".nav-tabs LI.active").data("jalios-tab-name");
          return;
        }
      }
    });

    // Add common parameters to ajax queries
    $.ajaxPrefilter(function(options, originalOptions, jqXHR) {
      if (originalOptions.explorer) {
        let newData = $.extend(originalOptions.data, { collaborativeSpaceApplicationDisplay : originalOptions.explorer.isCollaborativeSpaceApplicationDisplay() });
        
        const params = new URLSearchParams()
        Object.entries(newData).forEach(([key, value]) => {
          if (Array.isArray(value)) {
            value.forEach(value => params.append(key, value.toString()))
          } else if (value) {
            params.append(key, value.toString())
          }
        });
        options.data = params.toString();
      }
    });

    // Clean up menu dividers
    $('#explorer-dropdown-publish .divider:first-child').remove();
    $('#explorer-dropdown-publish .divider:last-child').remove();
    $('#explorer-dropdown-publish .divider + .divider').remove();
    $('#explorer-dropdown-publish  LI:not(.divider):empty').remove();

    EXPLORER.refreshNoResultDisplay();

    window.addEventListener('popstate', function (event) {
      // If sidebar is opened, only close it
      if ($(".explorer-preview-sidebar.is-open").exists()) {
        event.stopPropagation();
        event.preventDefault();
        EXPLORER_PREVIEW_SIDEBAR.close({});
        return;
      }

      let formData = ExplorerUtils.getExplorerFormData(false);
      formData.append(
        'explorerCurrentCategory',
        event.state && event.state.explorerCurrentCategory ? event.state.explorerCurrentCategory : 'root'
      );

      let scrollToPreviousElementId = EXPLORER.data.currentCategoryId;
      let isScrollToFolderEnabled = $('.app-body').data('jalios-explorer-app-scroll-to-previous-folder');

      $('.app-body').refresh({
        nohistory: true,
        noscroll: true,
        params: ExplorerUtils.getFormDataAsJsonObject(formData),
        callback: function () {
          EXPLORER.refreshNoResultDisplay();
          EXPLORER.data.currentCategoryId =
            event.state && event.state.explorerCurrentCategory ? event.state.explorerCurrentCategory : 'root';
          ExplorerUtils.clearSelection();
          ExplorerUtils.hideExplorerLoading();
          $.plugin.ExplorerApp.triggerEvent('movedToCategory', {
            categoryId:
              event.state && event.state.explorerCurrentCategory ? event.state.explorerCurrentCategory : 'root'
          });
          $('.sidebar-categorytree-item').removeClass('active');
          if (event.state && event.state.explorerCurrentCategory) {
            $('.sidebar-categorytree-item[data-jalios-data-id=' + event.state.explorerCurrentCategory + ']').addClass(
              'active'
            );
            document.title = $(
              '.sidebar-categorytree-item[data-jalios-data-id=' + event.state.explorerCurrentCategory + ']'
            )
              .find('.js-title')
              .text();
          } else {
            $('.sidebar-categorytree-item.is-root-access').addClass('active');
            document.title = initialPageTitle;
          }

          if (isScrollToFolderEnabled) {
            let $scrollToPreviousElement = $('.explorer-item[data-jalios-data-id=' + scrollToPreviousElementId + ']');
            if ($scrollToPreviousElement.exists()) {
              $([document.documentElement, document.body]).animate(
                {
                  scrollTop: $scrollToPreviousElement.offset().top - 104
                },
                0
              );
            } else {
              $([document.documentElement, document.body]).animate(
                {
                  scrollTop: $('.app-explorer').offset().top - 104
                },
                0
              );
            }
          }
        }
      });
    });
    
    $(document).on('input change propertychange', '.explorer-search-input', function (event) {
      if ($(this).val().length > 0) {
        $('.explorer-search-input-wrapper .btn-remove').removeClass('hide');
      } else {
        $('.explorer-search-input-wrapper .btn-remove').addClass('hide');
      }
    });

    $('.app-sidebar').resizable({
      handles: 'e'
    });

    $(document).on('hidden.jalios.sidebar', function (event) {
      let $siteSidebar = $(event.target);
      if (!$siteSidebar.hasClass('explorer-preview-sidebar')) {
        return;
      }
      $('.explorer-preview-sidebar .sidebar-tab-content').html('');
      $('.explorer-preview-sidebar .sidebar-tab-content').displayLoadingWaves();
    });
  });
})(window.jQuery);
